;************************************************
;* LAB3.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        1 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab configures Timer1 to use an external*
;* 32KHz crystal and operate asynch. to the main*
;* oscillator.  This will be used as a real time*
;* clock to keep track of minutes & seconds.    *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Defines external functions
	extern	InitLCD
	extern	putcLCD
	extern	clrLCD
	extern	L1homeLCD
	extern	L2homeLCD

; Variables used in this program;
	UDATA
Secs	res	1
Mins	res	1
SecsH	res	1
SecsL	res	1
MinsH	res	1
MinsL	res	1
_WREG	res	1
_STATUS	res	1

; Locates startup code @ the reset vector
STARTUP	code
	nop			;0x0000
	goto	Start		;0x0001
	nop			;0x0002
	nop			;0x0003
	goto	ISR		;0x0004

; Locates main code
PROG1	code
Start
	clrf	Secs		; Initialize the variables
	clrf	Mins
	call	InitLCD		; Initialize the LCD

;************************************************
;* Place your code here to do the following:    *
;*   - Initialize Timer1 with 8000h             *
;*   - Configure Timer1                         *
;*       - Prescale 1:1                         *
;*       - Enable Oscillator                    *
;*       - DO NOT SYNCHRONIZE                   *
;*       - External Clock Source                *
;*       - Enable Timer1                        *
;*   - Clear Timer1 Overflow Interrupt Flag     *
;*   - Enable Timer1 Overflow Interrupt         *
;*   - Enable Peripheral Interrupts             *
;*   - Enable Global Interrupts                 *
;************************************************
	banksel TMR1H
	movlw	0x80
	movwf	TMR1H
	clrf	TMR1L
	movlw	B'00001111'
	movwf	T1CON
	bcf	PIR1,TMR1IF
	banksel	PIE1
	bsf	PIE1,TMR1IE
	banksel	INTCON
	bsf	INTCON,PEIE
	bsf	INTCON,GIE
;************************************************
;* End of your code                             *
;************************************************
	call	InitDisplay	; Initializes display
				; contents
Main
	nop
	goto	Main


;************************************************
;* ISR - Interrupt Service Routine for Timer1   *
;* This code must first save WREG and STATUS,   *
;* update TMR1H, Secs, Mins, and the LCD panel. *
;************************************************
ISR
Push
	movwf	_WREG		; Save WREG
	swapf	STATUS,W	; Save STATUS
	movwf	_STATUS
;************************************************
;* Place your code here to do the following:    *
;*   - Clear Timer1 Overflow Interrupt Flag     *
;*   - Write 80h to TMR1H                       *
;*   - Increment Secs                           *
;*   - If Secs = 60 decimal                     *
;*        - Clear Secs                          *
;*        - Increment Mins                      *
;*   - Call DisplayTime to update LCD display   *
;************************************************
	bcf	PIR1,TMR1IF
	movlw	0x80
	movwf	TMR1H
	incf	Secs,F
	movlw	.60
	subwf	Secs,W
	btfss	STATUS,Z
	goto	ISRHere
	clrf	Secs
	incf	Mins,F
ISRHere
	call	DisplayTime

;************************************************
;* End of your code                             *
;************************************************
Pop
	swapf	_STATUS,W	; Restore STATUS
	movwf	STATUS
	swapf	_WREG,F		; Restore WREG w/o
	swapf	_WREG,W		; affecting STATUS
	retfie	


;************************************************
;* DisplayTime - Moves cursor to home of line 1,*
;* calls Bin2BCD to get ASCII equivalents of    *
;* Secs & Mins, and then writes them to the LCD.*
;************************************************
DisplayTime
	call	L1homeLCD	; Goto to home line 1
	call	Bin2BCD		; Call binary 2 bcd conversion
	movf	MinsH,W		; Write minutes high byte
	call	putcLCD		; to display
	movf	MinsL,W		; Write minutes low byte
	call	putcLCD		; to display
	movlw	':'		; Write ':' to display
	call	putcLCD
	movf	SecsH,W		; Write seconds high byte
	call	putcLCD		; to display
	movf	SecsL,W		; Write seconds low byte
	call	putcLCD		; to display
	return


;************************************************
;* Bin2BCD - converts Secs & Mins to ASCII      *
;* equivalents.  Stored in SecsH:SecsL and      *
;* MinsH:MinsL.                                 *
;************************************************
Bin2BCD
	clrf	MinsH		; Initialize variables
	movf	Mins,W		; for minutes
	movwf	MinsL
Mtenth				; Loop to subtract 10
	movlw	.10		; from low byte and
	subwf	MinsL,W		; increment high byte
	btfss	STATUS,C	; while low byte is > 0
	goto	Next
	movwf	MinsL
	incf	MinsH,F
	goto	Mtenth
Next
	clrf	SecsH		; Initialize variables
	movf	Secs,W		; for seconds
	movwf	SecsL
Stenth				; Loop to subtract 10
	movlw	.10		; from low byte and
	subwf	SecsL,W		; increment high byte
	btfss	STATUS,C	; while low byte is > 0
	goto	Over
	movwf	SecsL
	incf	SecsH,F
	goto	Stenth
Over
	movlw	0x30		; Add 0x30 ('0') to all
	addwf	MinsH,F		; variables to convert
	addwf	MinsL,F		; a number 0 - 9 to an
	addwf	SecsH,F		; ASCII equivalent
	addwf	SecsL,F		; '0' to '9'
	return

;************************************************
;* InitDisplay - initializes the message on the *
;* LCD display to 00:00.                        *
;************************************************
InitDisplay
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	call	clrLCD		; Clears display
	movlw	'0'		; Writes 00:00 to display
	call	putcLCD
	movlw	'0'
	call	putcLCD
	movlw	':'
	call	putcLCD
	movlw	'0'
	call	putcLCD
	movlw	'0'
	call	putcLCD
	return

	end
