;************************************************
;* LAB6.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        5 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab uses a potentiometer connected to   *
;* channel 0 of A/D to set the duty cycle of    *
;* PWM module which in turn drives a motor.     *
;* The upper 8-bits of the A/D conversion are   *
;* used as the upper 8-bits of the PWM duty     *
;* cycle.  The A/D result should be displayed   *
;* on the LCD display.                          *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Defines external functions
	extern	InitLCD
	extern	putcLCD
	extern	clrLCD
	extern	L1homeLCD
	extern	L2homeLCD

; Variables used in this program;
	UDATA
Pot	res	1
PotU	res	1
PotH	res	1
PotL	res	1
Count	res	1

; Locates startup code @ the reset vector
STARTUP	code
	nop
	goto	Start

; Locates main code
PROG1	code
Start
	call	InitLCD		; Initialize the LCD
;************************************************
;* Place your code here to do the following:    *
;*   - Initialize the A/D converter             *
;*        - Left justified                      *
;*        - 3 analog channels, 0 references     *
;*          (see Table 11-2 on page 122)        *
;*        - Fosc/8 clock source                 *
;*        - Channel 0                           *
;*        - Enable A/D                          *
;*   - Initialize the CCP1 Module               *
;*        - Set the period to FFh               *
;*        - Make RC2 an output                  *
;*        - Configure CCP1 for PWM              *
;*   - Initialize Timer2                        *
;*        - 1:1 Postscale                       *
;*        - 1:1 Prescale                        *
;*        - Enable the timer                    *
;************************************************
	bcf	STATUS,RP1
	bsf	STATUS,RP0
	movlw	0x04
	movwf	ADCON1
	bcf	STATUS,RP0
	movlw	B'01000001'
	movwf	ADCON0

	bsf	STATUS,RP0
	movlw	0xff
	movwf	PR2
	bcf	TRISC,2
	bcf	STATUS,RP0
	movlw	B'00001111'
	movwf	CCP1CON
	movlw	B'00000100'
	movwf	T2CON
;************************************************
;* End of your code                             *
;************************************************

Main
;************************************************
;* Place your code here to do the following:    *
;*   - Call routine to delay for 20us           *
;*   - Start an A/D conversion                  *
;*   - Wait for the conversion to complete      *
;*   - Save the result in both the variable Pot *
;*     and the duty cycle register CCPR1L       *
;************************************************
	call	Delay20		; Delay for 20us
	bsf	ADCON0,GO	; Start A/D conversion
WaitADC
	btfsc	ADCON0,GO	; Wait for the conversion
	goto	WaitADC		; to end
	movf	ADRESH,W	; Save result in Pot
	movwf	Pot
	movwf	CCPR1L		; Also in Duty cycle
;************************************************
;* End of your code                             *
;************************************************

	call	Bin2BCD		; Convert binary to BCD
	call	L1homeLCD	; Goto LCD line 1 home
	movf	PotU,W		; Write ASCII result
	call	putcLCD		; to LCD
	movf	PotH,W
	call	putcLCD
	movf	PotL,W
	call	putcLCD
	goto	Main

;************************************************
;* Bin2BCD - converts Pot to ASCII equivalent   *
;* Full three digit result is stored in         *
;* PotU:PotH:PotL.                              *
;************************************************
Bin2BCD
	clrf	PotU		; Initialize variables
	clrf	PotH
	movf	Pot,W
	movwf	PotL
Ptenth				; Subtract decimal 10
	movlw	.10		; from the binary and
	subwf	PotL,W		; increment high byte
	btfss	STATUS,C	; until low is < 10
	goto	Phund
	movwf	PotL
	incf	PotH,F
	goto	Ptenth
Phund				; Subtract decimal 10
	movlw	.10		; from the binary and
	subwf	PotH,W		; increment upper byte
	btfss	STATUS,C	; until high is < 10
	goto	Over
	movwf	PotH
	incf	PotU,F
	goto	Phund
Over
	movlw	0x30		; Adjust BCD numbers to
	addwf	PotU,F		; ASCII by adding 30h
	addwf	PotH,F
	addwf	PotL,F
	return

;************************************************
;*  Delay20 - delay for exactly 20us including  *
;*  the call and the return instructions.       *
;************************************************
Delay20				; 2 Tcy for call
	movlw	0x05		; 1 Tcy
	movwf	Count		; \
D20Loop				;  \ 3 * 5
	decfsz	Count,F		;  / or 15 Tcy
	goto	D20Loop		; /
	return			; 2 Tcy for return

	end
