;************************************************
;* LAB7.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        5 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab uses a potentiometer connected to   *
;* channel 0 of A/D to set the duty cycle of    *
;* PWM module which in turn drives a motor.     *
;* The upper 8-bits of the A/D conversion are   *
;* used as the upper 8-bits of the PWM duty     *
;* cycle.  The A/D result should be displayed   *
;* on the LCD display. The other CCP module is  *
;* used in capture mode and reads the RPM of    *
;* the motor using an optoreflector and a       *
;* slotted wheel mounted on the motor.          *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Defines external functions
	extern	InitLCD
	extern	putcLCD
	extern	clrLCD
	extern	L1homeLCD
	extern	L2homeLCD
	extern	FXD3216U

; Variables used in this program, but found in another file
	extern	AARGB0
	extern	AARGB1
	extern	AARGB2
	extern	AARGB3
	extern	BARGB0
	extern	BARGB1

; Variables used in this program
	UDATA
Pot	res	1
PotU	res	1
PotH	res	1
PotL	res	1
Count	res	1
CapH	res	1
CapL	res	1
Hbyte	res	1
Lbyte	res	1
TempWREG	res	1
TempSTATUS	res	1
R0	res	1
R1	res	1
R2	res	1
temp	res	1


; Locates startup code @ the reset vector
STARTUP	code
	nop			; Reset vector
	goto	Start
	nop
	nop
	goto	ISR		; Interrupt vector

; Locates main code
PROG1	code
Start
	call	InitLCD		; Initialize the LCD

	bsf	STATUS,RP0
	movlw	0x04		; Initialize the A/D
	movwf	ADCON1		; same as Lab6
	movlw	0xff		; Set period register to FFh
	movwf	PR2
	bcf	TRISC,2		; Make RC2 an output
	bcf	STATUS,RP0	; Initialize the A/D
	movlw	B'01000001'	; same as Lab6
	movwf	ADCON0
	movlw	B'00001111'	; Initialize CCP1 same
	movwf	CCP1CON		; as Lab6 (PMW mode)
	movlw	B'00000100'	; Initialize Timer2
	movwf	T2CON		; same as Lab6

;************************************************
;* Place your code here to do the following:    *
;*   - Initialize Timer1                        *
;*        - 1:1 Prescale                        *
;*        - Oscillator Off                      *
;*        - Internal Clock source               *
;*        - Enable the timer                    *
;*   - Initialize the CCP2 Module               *
;*        - Configure CCP2 for Capture          *
;*        - Capture every rising edge           *
;*   - Enable Timer1 overflow interrupt         *
;*   - Enable Capture2 interrupt                *
;*   - Enable Peripheral interrupts             *
;*   - Enable global interrupts                 *
;************************************************
	movlw	B'00000001'
	movwf	T1CON
	movlw	B'00000101'
	movwf	CCP2CON
	bsf	STATUS,RP0
	bsf	PIE1,TMR1IE
	bsf	PIE2,CCP2IE
	bcf	STATUS,RP0
	bsf	INTCON,PEIE
	bsf	INTCON,GIE
;************************************************
;* End of your code                             *
;************************************************

Main
	call	Delay20		; Delay for 20us
	bsf	ADCON0,GO	; Start A/D conversion
WaitADC
	btfsc	ADCON0,GO	; Wait for conversion to
	goto	WaitADC		; complete
	movf	ADRESH,W	; Save A/D result in
	movwf	Pot		; variable Pot and
	movwf	CCPR1L		; PWM duty cyle register
	call	Bin2BCD		; Convert binary to BCD
	call	L1homeLCD	; Goto LCD line 1 home
	movf	PotU,W		; Write A/D result to
	call	putcLCD		; LCD display
	movf	PotH,W
	call	putcLCD
	movf	PotL,W
	call	putcLCD

;************************************************
;* Place your code here to do the following:    *
;*   - Initialize the registers to perform a    *
;*     division of 01C9C380h by the capture     *
;*     value.  AARG holds 32-bit number above,  *
;*     with AARGB0 the MSbyte and AARGB3 the    *
;*     LSbyte.                                  *
;*   - Move the contents of CapH:CapL into the  *
;*     variables BARGB0:BARGB1.                 *
;*   - Call the routine FXD3216U to divide      *
;*     AARG by BARG                             *
;************************************************
CalculateRPM
	movlw	0x01
	movwf	AARGB0
	movlw	0xC9
	movwf	AARGB1
	movlw	0xC3
	movwf	AARGB2
	movlw	0x80
	movwf	AARGB3
	movf	CapH,W
	movwf	BARGB0
	movf	CapL,W
	movwf	BARGB1
	call	FXD3216U
;************************************************
;* End of your code                             *
;************************************************
	movf	AARGB2,W
	movwf	Hbyte
	movf	AARGB3,W
	movwf	Lbyte
	call	DisplayRPM
	goto	Main


;************************************************
;*  Delay20 - delay for exactly 20us including  *
;*  the call and the return instructions.       *
;************************************************
Delay20				; 2 Tcy for call
	movlw	0x05		; 1 Tcy
	movwf	Count		; \
D20Loop				;  \ 3 * 5
	decfsz	Count,F		;  / or 15 Tcy
	goto	D20Loop		; /
	return			; 2 Tcy for return


ISR
Push
	movwf	TempWREG	; Save WREG
	swapf	STATUS,W	; Save STATUS
	movwf	TempSTATUS

Capture2
	btfss	PIR2,CCP2IF
	goto	Timer1
;************************************************
;* Place your code here to do the following:    *
;*   - Clear Timer1 (TMR1H:TMR1L)               *
;*   - Save capture values CCPR2H:CCPR2L in     *
;*     CapH:CapL                                *
;*   - Clear the Capture2 interrupt flag        *
;*   - Clear the Timer1 Overflow flag           *
;************************************************
	clrf	TMR1L
	clrf	TMR1H
	movf	CCPR2L,W
	movwf	CapL
	movf	CCPR2H,W
	movwf	CapH
	bcf	PIR2,CCP2IF
	bcf	PIR1,TMR1IF
;************************************************
;* End of your code                             *
;************************************************
	goto	Pop
Timer1
	clrf	CapL
	clrf	CapH
	bcf	PIR1,TMR1IF
Pop
	swapf	TempSTATUS,W	; Restore STATUS
	movwf	STATUS
	swapf	TempWREG,F	; Restore WREG w/o
	swapf	TempWREG,W	; affecting STATUS
	retfie

;************************************************
;* DisplayRPM - calculates BCD from binary and  *
;* displays the result on the LCD display.      *
;************************************************
DisplayRPM
	call	Bin2BCD16	; Converts 16-bit binary
				; into 3-byte packed BCD
				; R0 MSbyte, R2 LSbyte
	movf	R1,W		; Only convert the upper
	movwf	Pot
	call	Bin2BCD		; Convert the mid byte to BCD
	movlw	' '		; Add some space from the
	call	putcLCD		; A/D result
	movlw	' '
	call	putcLCD
	movf	R0,F		; If R0 is not 0 then
	btfsc	STATUS,Z	; the RPM is incorrect
	goto	TrueRPM
	movlw	'0'		; Display 0's if RPM is
	movwf	PotH		; incorrect
	movwf	PotL
TrueRPM
	movf	PotH,W		; Write the RPM to the
	call	putcLCD		; LCD display
	movf	PotL,W		; Only hundreds of RPM
	call	putcLCD		; are shown
	movlw	'0'		; The tens and ones of
	call	putcLCD		; RPM are always shown
	movlw	'0'		; as 0's
	call	putcLCD
	return

;************************************************
;* Bin2BCD16 - Converts a 16-bit binary number  *
;* in Hbyte:Lbyte into a 3 byte packed BCD      *
;* number in R0:R1:R2.                          *
;* R2 holds the ones and tens digits            *
;* R1 holds the hundreds and thousands digits   *
;* If R0 is anything but 0 then the RPM is      *
;* incorrect due to Timer1 overflow during      *
;* measurement.                                 *
;************************************************
Bin2BCD16
	bcf	STATUS,C	; Initialize variables
	movlw	D'16'
	movwf	Count
	clrf	R0
	clrf	R1
	clrf	R2
loop16
	rlf	Lbyte, F
	rlf	Hbyte, F
	rlf	R2, F
	rlf	R1, F
	rlf	R0, F
	decfsz	Count, F
	goto	adjDEC
	retlw	0
adjDEC
	movlw	R2
	movwf	FSR
	call	adjBCD
	movlw	R1
	movwf	FSR
	call	adjBCD
	movlw	R0
	movwf	FSR
	call	adjBCD
	goto	loop16
adjBCD
	movlw	3
	addwf	0,W
	movwf	temp
	btfsc	temp,3
	movwf	0
	movlw	30
	addwf	0,W
	movwf	temp
	btfsc	temp,7
	movwf	0
	retlw	0


;************************************************
;* Bin2BCD - converts Pot to ASCII equivalent   *
;* Full three digit result is stored in         *
;* PotU:PotH:PotL.                              *
;************************************************
Bin2BCD
	clrf	PotU		; Initialize variables
	clrf	PotH
	movf	Pot,W
	movwf	PotL
Ptenth				; Subtract decimal 10
	movlw	.10		; from the binary and
	subwf	PotL,W		; increment high byte
	btfss	STATUS,C	; until low is < 10
	goto	Phund
	movwf	PotL
	incf	PotH,F
	goto	Ptenth
Phund				; Subtract decimal 10
	movlw	.10		; from the binary and
	subwf	PotH,W		; increment upper byte
	btfss	STATUS,C	; until high is < 10
	goto	Over
	movwf	PotH
	incf	PotU,F
	goto	Phund
Over
	movlw	0x30		; Adjust BCD numbers to
	addwf	PotU,F		; ASCII by adding 30h
	addwf	PotH,F
	addwf	PotL,F
	return


	end
