;************************************************
;* LAB8.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        5 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab uses the USART in 9-bit addressable *
;* mode in a RS-485 network.  The master has an *
;* address of 0 and the slave addresses start   *
;* at 80h.  The master transmits the slave      *
;* address followed by a data byte.  The slave  *
;* writes the data byte to PORTD LEDs and sends *
;* the data byte back to the master.            *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Variable to hold the slave address
SLVADD	EQU	0x20


; Reset vector
	ORG	0x0000
	nop
	bcf	STATUS,RP1           ;Ports in Bank 0
	bcf	STATUS,RP0  
	clrf	PORTD
	clrf	PORTC
	bsf	STATUS,RP0           ;  TRIS regs in Bank 1
	clrf	TRISD
	bcf	PORTC,0
	bcf	STATUS,RP0

;************************************************
;* Place your code here to do the following:    *
;*   - Initialize the USART                     *
;*       - 9600 Baud @ 4MHz                     *
;*       - 9-bit transmit                       *
;*       - Transmit enabled                     *
;*       - Asynchronous                         *
;*       - High speed baud rate select          *
;*       - Serial port enabled                  *
;*       - 9-bit reception                      *
;*       - Enable continuous receive            *
;*       - Enable address detect                *
;*   - Initialize your slave address by writing *
;*     your address to the SLVADD register      *
;************************************************
	bsf	STATUS,RP0
	movlw	D'25'                ; SPBRG VALUE FOR 9600 BAUD @ 4MHZ
	movwf	SPBRG
	clrf	TXSTA                ; CLEAR AND SETUP TXSTA
	bsf	TXSTA,BRGH           ; ENABLE HIGH BAUD RATE
	bsf	TXSTA,TXEN           ; ENABLE SERIAL TRANSMISSION
	bsf	TXSTA,TX9            ; ENABLE 9-BIT TRANSMISSION
	bcf	STATUS,RP0
	clrf	RCSTA                ; CLEAR AND SETUP RCSTA 
	bsf	RCSTA,CREN           ; ENABLE CONTINUOUS RECEPTION
	bsf	RCSTA,SPEN           ; ENABLE SERIAL PORT
	bsf	RCSTA,RX9            ; ENABLE 9-BIT RECEPTION
	bsf	RCSTA,ADDEN          ; ENABLE 9-BIT ADDRESS INTERRUPT
   
	bcf	STATUS,RP0
	movlw	0x9E
	movwf	SLVADD
;************************************************
;* End of your code                             *
;************************************************

Main
;************************************************
;* Place your code here to do the following:    *
;*   - Enable address detect                    *
;*   - Call RXPOLL to wait for a reception      *
;*   - When reception occurs                    *
;*       - Check if it is equal to your slave   *
;*         address                              *
;*       - If not go back to Main               *
;*       - Otherwise disable address detect     *
;*   - Call RXPOLL to wait for nex reception    *
;*   - Write received data to PORTD             *
;************************************************
	bcf	STATUS,RP0
	bsf	RCSTA,ADDEN
	call	RXPOLL               ; LOOP HERE UNTIL WE GET AN ADDRESS INTERRUPT.
	movf	RCREG,W              ; GET THE ADDRESS VALUE IN RCREG.
	subwf	SLVADD,W             ; SUBTRACT THE VALUE FOR OUR SLAVE ADDRESS.
	btfss	STATUS,Z                          ; DO WE HAVE AN ADDRESS MATCH?
	goto	Main                 ; NO, WE DIDN'T.
	bcf	RCSTA,ADDEN          ; YES, WE DID.  DISABLE ADDEN.
	call	RXPOLL               ; SIT HERE AND WAIT FOR THE DATA PACKET.
	movf	RCREG,W              ; GET THE DATA FROM THE USART.
	movwf	PORTD                ; AND PUT IT ON THE LED'S.
	                             ; OK, NOW IT'S TIME TO RESPOND BACK TO THE
        	                     ; MASTER.
;************************************************
;* End of your code                             *
;************************************************

	bsf	STATUS,RP0           
	bsf	TXSTA,TX9D           ; SET 9TH BIT FOR ADDRESS.
	bcf	STATUS,RP0
   
	bsf	PORTC,0              ; ENABLE MAX1482 OUTPUT.
	movlw	0                    ; MASTER WILL BE SET TO ADDRESS 0x100
	movwf	TXREG                ; SEND OUT ADDRESS. 
	call	TXPOLL               ; WAIT FOR USART SHIFT REG TO FINISH.
	bsf	STATUS,RP0 
	bcf	TXSTA,TX9D           ; CLEAR 9TH BIT FOR DATA.
	bcf	STATUS,RP0
	movf	PORTD,W              ; WRITE THE RECEIVED DATA VALUE
	movwf	TXREG                ; TO THE USART.
	call	TXPOLL               ; WAIT FOR USART SHIFT REG TO FINISH.
	bcf	PORTC,0              ; DISABLE MAX1482 OUTPUT.
	
	goto	Main	              ; OK, ALL DONE, START OVER
	
   
;************************************************
;* RXPOLL - This function polls the USART       *
;* receive interrupt flag and waits until a     *
;* data byte is available in RCREG.             *
;************************************************
RXPOLL
	bcf	STATUS,RP0
	btfss	PIR1,RCIF
	goto	RXPOLL
	return

;************************************************
;* TXPOLL - This function polls the TRMT flag   *
;* and waits until the USART has finished       * 
;* shifting the data byte out of the transmit   *
;* shift register.                              *
;************************************************
TXPOLL
	bsf	STATUS,RP0
TLOOP
	btfss	TXSTA,TRMT
	goto	TLOOP
	bcf	STATUS,RP0
	return   

	end
