   LIST P=16f877

#include <p16f877.inc>

#define  DE    PORTC,0


	EXTERN	InitLCD
	EXTERN	putcLCD
	EXTERN	clrLCD
	EXTERN	L1homeLCD
	EXTERN	L2homeLCD

	UDATA
COUNT0   RES   1                
COUNT1   RES   1   
COUNT2   RES   1
MSTADD   RES   1
XMTDAT   RES   1
SLVADD   RES   1
MSD      RES   1
LSD      RES   1
TEMP     RES   1


STARTUP CODE
	nop
	GOTO	START


PROG1	CODE 
START
        
   CALL     INIT_PORTS
   CALL     INIT_USART
   CALL     INIT_AD
   CALL     INIT_TMR1
   
   
   BANKSEL  TEMP                 ; Brief delay to allow LCD
   CLRF     TEMP                 ; to power up before initializing
   DECFSZ   TEMP,F
   GOTO     $ - 1
   CALL     InitLCD
   call	    clrLCD
   
   
MLOOP

   
   BANKSEL  ADCON0
   BSF      ADCON0,GO            ; Start an AD conversion
ADPOLL
   BTFSC    ADCON0,GO            ; Poll for the conversion to finish
   GOTO     ADPOLL
   
   BANKSEL  ADRESH
   RRF      ADRESH,F             ; Take 5 MSB's of result and use
   RRF      ADRESH,F             ; for slave address to be written to.
   RRF      ADRESH,F
   MOVLW    0x1F
   ANDWF    ADRESH,F
   MOVLW    0x80                 ; Add 0x80 for addresses 0x80 - 0x9f
   ADDWF    ADRESH,W
   BANKSEL  SLVADD
   MOVWF    SLVADD               ; Put result in SLVADD
   
   call     L1homeLCD
   movlw    'S'
   call     putcLCD
   movlw    'l'
   call     putcLCD
   movlw    'v'
   call     putcLCD
   movlw    '='
   call     putcLCD
   
   movf     SLVADD,W             ; Convert slave address to ASCII
   call     BIN2HEX              ; and write to LCD
   banksel  MSD
   movf     MSD,W
   call     HEX2ASC
   call     putcLCD
   banksel  LSD
   movf     LSD,W
   call     HEX2ASC
   call     putcLCD
   
        
   
   BANKSEL  PORTA
   BTFSC    PORTA,4
   GOTO     MLOOP                ; If a button press is detected,
KEYPOLL:                         ; poll for a release.  Otherwise, 
   BTFSS    PORTA,4              ; go back and do another AD conversion.
   GOTO     KEYPOLL
           
   
PUTADDR:                         ; This code writes address and data
   call     L2homeLCD            ; packet to a slave.
   call     blankLCD
   call     L2homeLCD
   movlw    'S'
   call     putcLCD
   movlw    'n'
   call     putcLCD
   movlw    'd'
   call     putcLCD
   movlw    '='
   call     putcLCD
   
   banksel  XMTDAT               ; Convert data to be tranmsitted
   movf     XMTDAT,W             ; to ASCII and put it on the LCD
   call     BIN2HEX
   banksel  MSD
   movf     MSD,W
   call     HEX2ASC
   call     putcLCD
   banksel  LSD
   movf     LSD,W
   call     HEX2ASC
   call     putcLCD
   
   movlw    ' '
   call     putcLCD
   movlw    'R'
   call     putcLCD
   movlw    'e'
   call     putcLCD
   movlw    'c'
   call     putcLCD
   movlw    '='
   call     putcLCD         
     

   BANKSEL  TXSTA
   BSF      TXSTA,TX9D           ; SET 9TH BIT FOR ADDRESS
   
   BANKSEL  PORTC
   BSF      DE                   ; ENABLE MAX1482 OUTPUT
   BANKSEL  SLVADD
   MOVF     SLVADD,W             ; PUT THE CURRENT SLAVE'S ADDRESS IN W
   BANKSEL  TXREG
   MOVWF    TXREG                ; SEND OUT ADDRESS 
   CALL     TXPOLL               ; WAIT UNTIL DONE TRANSMITTING
   BCF      DE                   ; DISABLE MAX1482 OUTPUT
      
   BANKSEL  TXSTA 
   BCF      TXSTA,TX9D           ;  CLEAR 9TH BIT FOR DATA
   
   BANKSEL  XMTDAT
	MOVF     XMTDAT,W
	BANKSEL  PORTC
	BSF      DE                   ; SEND OUT THE DATA BYTE
	MOVWF    TXREG
   CALL     TXPOLL
   BCF      DE




RECDATA:                         ; THE FOLLOWING CODE LOOKS FOR THE
                                 ; RETURN PACKET FROM THE SLAVE
   
   BANKSEL  RCSTA   
   BSF      RCSTA,ADDEN          ;  SETUP FOR SLAVE RESPONSE
   
   BANKSEL  TMR1H
   MOVLW    0x80
   MOVWF    TMR1H
   BANKSEL  PIR1
   BCF      PIR1,TMR1IF
   
ADDRWAIT:
   BTFSS    PIR1,TMR1IF          ; RECEPTION WILL ABORT AFTER TIMEOUT PERIOD
   GOTO     CHECKRX
   GOTO     ABORT
CHECKRX:   
   BTFSS    PIR1,RCIF            ; LOOP HERE UNTIL WE GET AN ADDRESS INTERRUPT
	GOTO     ADDRWAIT             ; (9TH BIT SET)
	
	MOVF     RCREG,W              ; READ THE ADDRESS VALUE IN RCREG
         
	SKPZ                          ; DO WE HAVE AN ADDRESS MATCH?
	GOTO     RECDATA              ; NO, WE DIDN'T
	
	BCF      RCSTA,ADDEN          ; YES, WE DID.  DISABLE ADDEN

DATLOOP:
   BTFSS    PIR1,RCIF            ; SIT HERE AND WAIT FOR THE DATA PACKET
   GOTO     DATLOOP
      	
	MOVF     RCREG,W              ; GET THE DATA FROM THE USART
	call     BIN2HEX              ; Convert received data to ASCII and
	banksel  MSD                  ; display it on the LCD.
   movf     MSD,W
   call     HEX2ASC
   call     putcLCD
   banksel  LSD
   movf     LSD,W
   call     HEX2ASC
   call     putcLCD
	
	BANKSEL  PORTD                ; We're also using PORTD<0> as an error
	CLRF     PORTD                ; light, so clear it.
	
   GOTO     ALLDONE

ABORT:                           ; We didn't receive data from the slave.
   movlw    'E'                  ; So put the error message on the LCD.
   call     putcLCD
   movlw    'R'
   call     putcLCD
   movlw    'R'
   call     putcLCD
   movlw    '!'
   call     putcLCD
   BANKSEL  PORTD                ; Light up PORTD<0> as an error indicator
   BSF      PORTD,0
   

ALLDONE:
   BANKSEL  XMTDAT               ; Increment the data byte and start
   INCF     XMTDAT,F               ; over.
   
   GOTO     MLOOP



;-----------------------------SUBROUTINES-------------------------------

HEX2ASC:                         ; This routine takes hex values 
   BANKSEL  TEMP                 ; and coverts them to the
   MOVWF    TEMP                 ; ASCII equivalent
   MOVLW    0x0A
   SUBWF    TEMP,W
   BTFSC    STATUS,C
   GOTO     ISAF
IS09
   MOVLW    '0'
   ADDWF    TEMP,W
   RETURN
ISAF
   MOVLW    '7'
   ADDWF    TEMP,W
   RETURN


BIN2HEX:                         ; This routine takes a data byte
   BANKSEL  MSD                  ; and converts it to hex digits 
   CLRF     MSD                  ; stored in MSD and LSD.
   BANKSEL  LSD
   MOVWF    LSD
MSD_SUB:
   MOVLW    0x10
   SUBWF    LSD,W
   BTFSS    STATUS,C
   RETLW    0
   MOVWF    LSD
   INCF     MSD,F
   GOTO     MSD_SUB

INIT_PORTS:

   BANKSEL  PORTB
   MOVLW    0
   MOVWF    INTCON
   MOVWF    PORTB
   MOVWF    PORTC
   MOVWF    PORTD
   BANKSEL  XMTDAT
   MOVWF    XMTDAT
   MOVLW    0x80
   BANKSEL  MSTADD
   MOVWF    MSTADD
       
   MOVLW    0xFF
   BANKSEL  TRISB
   MOVWF    TRISB                ;  Port B all INPUTS
   BCF      OPTION_REG,NOT_RBPU  ;  ENABLE PULLUPS ON PORTB
   
   MOVLW    B'10010000'          ;  Configuration for TRISC
   MOVWF    TRISC
   
   CLRF     TRISD
   
   BANKSEL  PORTD
     
   RETURN
   
;---------------------------------------------------------------   

blankLCD:                        ; This routine puts 16 blank
   banksel  TEMP                 ; spaces on the LCD to blank
   movlw    0x10                 ; data without using the CLR
   movwf    TEMP                 ; command.
putSpace
   movlw    ' '
   call     putcLCD
   decfsz   TEMP,F
   goto     putSpace
   RETURN

INIT_USART:

   BANKSEL  SPBRG
   MOVLW    D'25'                ; SPBRG VALUE FOR 9600 BAUD @ 4MHZ
   MOVWF    SPBRG
   CLRF     TXSTA                ; CLEAR AND SETUP TXSTA
   BSF      TXSTA,BRGH           ; ENABLE HIGH SPEED BAUD RATE
   BSF      TXSTA,TXEN           ; ENABLE SERIAL TRANSMISSION
   BSF      TXSTA,TX9            ; ENABLE 9-BIT TRANSMISSION
       
   
   BANKSEL  RCSTA
   CLRF     RCSTA                ; CLEAR AND SETUP RCSTA
   BSF      RCSTA,CREN           ; ENABLE CONTINOUS RECEPTION
   BSF      RCSTA,SPEN           ; ENABLE SERIAL PORT
   BSF      RCSTA,RX9            ; ENABLE 9 BIT RECEPTION
         
   RETURN
   
;--------------------------------------------------------------
   
INIT_AD:

   BANKSEL  ADCON1
   MOVLW    B'00001110'          ; LEFT JUSTIFY, 1 ANALOG, 
   MOVWF    ADCON1               ; VREF = VDD, VSS
   BANKSEL  ADCON0               ;
   MOVLW    B'01000001'          ; FOSC/8, CH0, AD ON
   MOVWF    ADCON0

   RETURN

;-----------------------------------------------------------------------------

INIT_TMR1
   BANKSEL  T1CON
   BSF      T1CON,T1OSCEN
   BSF      T1CON,TMR1CS
   BSF      T1CON,TMR1ON
   RETURN


;-------------------------------------------------------------------------

RXPOLL
   BANKSEL  PIR1
RLOOP
   BTFSS    PIR1,RCIF
   GOTO     RLOOP
   RETURN

;-------------------------------------------------------------------------

TXPOLL
   BANKSEL  TXSTA   
TLOOP
   BTFSS    TXSTA,TRMT
   GOTO     TLOOP
   RETURN

;-------------------------------------------------------------------------
   


   END               ;END OF PROGRAM
