;************************************************
;* LAB9.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        5 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab calculates a checksum for the first *
;* 4K of program memory.  It stores it in       *
;* address 0FFFh.                               *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Defines external functions
	extern	InitLCD
	extern	putcLCD
	extern	clrLCD
	extern	L1homeLCD
	extern	L2homeLCD

; Variables used in this program in shared region
	UDATA_SHR
CheckH	res	1	;Checksum High byte
CheckL	res	1	;Checksum Low byte
AddrH	res	1	;Address High byte
AddrL	res	1	;Address Low byte

; Variables used in this program
	UDATA
DispHBUN	res	1	; High byte, upper nibble
DispHBLN	res	1	; High byte, lower nibble
DispLBUN	res	1	; Low byte, upper nibble
DispLBLN	res	1	; Low byte, lower nibble
Temp		res	1	; Temporary variable

; Locates startup code @ the reset vector
STARTUP	code
	nop
	goto	Start

; Locates main code
PROG1	code
Start
	call	InitLCD		; Initialize the LCD
	bsf	STATUS,RP0	;Bank 0
	bcf	STATUS,RP1
	clrf	CheckH		;Clear registers
	clrf	CheckL
	clrf	AddrH
	clrf	AddrL

Main
;************************************************
;* Place your code here to do the following:    *
;*   - Call ReadProgMem                         *
;*   - Go to Bank 2                             *
;*   - Do a 16-bit add of EEDATH:EEDATA with    *
;*     CheckH:CheckL                            *
;*   - Do a 16-bit increment of AddrH:AddrL     *
;************************************************
	call	ReadProgMem	;Read a byte of prog mem
	bcf	STATUS,RP0	;Bank 2
	bsf	STATUS,RP1
	movf	EEDATA,W	;Add values to
	addwf	CheckL,F	;checksum
	btfsc	STATUS,C
	incf	CheckH,F
	movf	EEDATH,W
	addwf	CheckH,F
	incf	AddrL,F		;Incrment address
	btfsc	STATUS,Z
	incf	AddrH,F
;************************************************
;* End of your code                             *
;************************************************

	movlw	0x0f		;Check for address
	subwf	AddrH,W		;0EFFh
	btfss	STATUS,Z
	goto	Main
	movlw	0xff
	subwf	AddrL,W
	btfss	STATUS,Z
	goto	Main

;************************************************
;* Place your code here to do the following:    *
;*   - 2's complement CheckH:CheckL             *
;************************************************
	comf	CheckH,F	;Take 2's complement
	comf	CheckL,F	;of checksum
	incf	CheckL,F
	btfsc	STATUS,Z
	incf	CheckH,F
;************************************************
;* End of your code                             *
;************************************************

	call	WriteProgMem	;Write checksum to mem
	call	ReadProgMem	;Read checksum from mem
	bcf	STATUS,RP0
	bsf	STATUS,RP1
	movf	EEDATA,W
	movwf	CheckL
	movf	EEDATH,W
	movwf	CheckH
	call	DisplayChecksum

	goto	$		;Loop forever


;************************************************
;* ReadProgMem - reads the program memory       *
;* contents located at address in AddrH:AddrL.  *
;* Result is in EEDATH:EEDATA.                  *
;************************************************
ReadProgMem			;Reads program mem
	bcf	STATUS,RP0	;Bank 2
	bsf	STATUS,RP1
	movf	AddrH,W		;Write address to
	movwf	EEADRH		;EEADRH:EEADR
	movf	AddrL,W
	movwf	EEADR
	bsf	STATUS,RP0	;Bank 3
	bsf	EECON1,EEPGD	;Point to Prog mem
	bsf	EECON1,RD	;Start Read
	nop			;Required NOPs
	nop
	return

;************************************************
;* WriteProgMem - Writes the contents of        *
;* CheckH:CheckL to the program memory address  *
;* in AddrH:AddrL.                              *
;************************************************
WriteProgMem			;Write to program mem
	bcf	STATUS,RP0	;Bank 2
	bsf	STATUS,RP1
	movf	AddrH,W		;Write address to
	movwf	EEADRH		;EEADRH:EEADR
	movf	AddrL,W
	movwf	EEADR
	movf	CheckH,W	;Write data to
	movwf	EEDATH		;EEDATH:EEDATA
	movf	CheckL,W
	movwf	EEDATA
	bsf	STATUS,RP0	;Bank 3
	bsf	EECON1,EEPGD	;Point to Prog mem
	bsf	EECON1,WREN	;Enable Writes
	movlw	0x55		;Required write
	movwf	EECON2		;sequence
	movlw	0xaa
	movwf	EECON2
	bsf	EECON1,WR	;Start Write
	nop			;Required NOPs
	nop
	bcf	EECON1,WREN	;Disable Writes
	return

;************************************************
;* DisplayChecksum - Displays the calculated    *
;* checksum read from location 0EFFh in HEX.    *
;************************************************
DisplayChecksum
	bcf	STATUS,RP0	;Go to Bank 0
	bcf	STATUS,RP1
	call	clrLCD		;Clear the LCD
	call	L1homeLCD	;Goto LCD line 1 home
	call	Bin2HEX		;Convert binary to HEX
	movf	DispHBUN,W	;Write checksum to LCD
	call	putcLCD
	movf	DispHBLN,W
	call	putcLCD
	movf	DispLBUN,W
	call	putcLCD
	movf	DispLBLN,W
	call	putcLCD
	return

;************************************************
;* Bin2HEX - Takes the 16-bit value in          *
;* CheckH:CheckL and converts it to ASCII chars *
;* in DispHBUN:DispHBLN:DispLBUN:DispLBLN.      *
;************************************************
Bin2HEX
	swapf	CheckH,W
	andlw	0x0f
	call	HEXTable
	movwf	DispHBUN
	movf	CheckH,W
	andlw	0x0f
	call	HEXTable
	movwf	DispHBLN
	swapf	CheckL,W
	andlw	0x0f
	call	HEXTable
	movwf	DispLBUN
	movf	CheckL,W
	andlw	0x0f
	call	HEXTable
	movwf	DispLBLN
	return
;************************************************
;* HEXTable - Adds offset in WREG to program    *
;* counter and returns ASCII equivalent of the  *
;* offset in HEX.                               *
;************************************************
HEXTable
	addwf	PCL,F
	retlw	'0'
	retlw	'1'
	retlw	'2'
	retlw	'3'
	retlw	'4'
	retlw	'5'
	retlw	'6'
	retlw	'7'
	retlw	'8'
	retlw	'9'
	retlw	'A'
	retlw	'B'
	retlw	'C'
	retlw	'D'
	retlw	'E'
	retlw	'F'



	end
