;************************************************
;* LAB5.ASM                                     *
;************************************************
;* MCU201 Workshop                              *
;* Written by:  Rodger Richey                   *
;*              Principal Applications Engr.    *
;*              Microchip Technology Inc.       *
;* Date:        1 April 1999                    *
;* Revision:	1                               *
;************************************************
;* This lab makes A/D conversions on a          *
;* thermistor in a bridge configuration.  The   *
;* result are sent to a calibration table that  *
;* returns the calibrated temperature that is   *
;* then converted to ASCII and written to LCD.  *
;************************************************
	list p = 16f877
	include <p16f877.inc>
	__CONFIG _BODEN_OFF&_CP_OFF&_WRT_ENABLE_ON&_PWRTE_ON&_WDT_OFF&_XT_OSC&_DEBUG_OFF&_CPD_OFF&_LVP_OFF

; Defines external functions
	extern	InitLCD
	extern	putcLCD
	extern	clrLCD
	extern	L1homeLCD
	extern	L2homeLCD

; Variables used in this program;
	UDATA
Temp	res	1
TempH	res	1
TempL	res	1
Count	res	1

; Locates startup code @ the reset vector
STARTUP	code
	nop
	goto	Start

; Locates main code
PROG1	code
CalTable
		movwf	Temp		; Save A/D resutl in Temp
		movlw	0x66		; Subtract out offset from
		subwf	Temp,w		; reading
		addwf	PCL,f		; Add compensated result to PCL
 dt	0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5
 dt	6,6,6,7,7,7,8,8,8,9,9,0xa,0xa,0xb,0xb,0xc,0xc
 dt	0xd,0xd,0xe,0xe,0xf,0xf,0xf,0x10,0x10,0x10
 dt	0x11,0x11,0x12,0x12,0x12,0x13,0x13,0x14,0x14,0x14
 dt	0x15,0x15,0x15,0x16,0x16,0x16,0x17,0x17,0x18,0x18
 dt	0x19,0x19,0x1a,0x1a,0x1b,0x1b,0x1c,0x1c,0x1c
 dt	0x1d,0x1d,0x1d,0x1e,0x1e,0x1f,0x1f,0x20,0x20
 dt	0x21,0x21,0x22,0x22,0x23,0x23,0x24,0x24,0x25,0x25
 dt	0x26,0x26,0x27,0x28,0x29,0x29,0x2a,0x2a,0x2b,0x2b
 dt	0x2c,0x2d,0x2e,0x2f,0x30,0x32


Start
	call	InitLCD		; Initializes the LCD
;************************************************
;* Place your code here to do the following:    *
;*   - Initialize the A/D converter             *
;*        - Left justified                      *
;*        - 3 analog channels, 0 references     *
;*          (see Table 11-2 on page 122)        *
;*        - Fosc/8 clock source                 *
;*        - Channel 1                           *
;*        - Enable A/D                          *
;************************************************


;************************************************
;* End of your code                             *
;************************************************

Main
;************************************************
;* Place your code here to do the following:    *
;*   - Call routine to delay for 20us           *
;*   - Start an A/D conversion                  *
;*   - Wait for the conversion to complete      *
;*   - Send the A/D result (upper 8 bits) to    *
;*     calibration table CalTable               *
;*   - Store the result in variable Temp        *
;************************************************


;************************************************
;* End of your code                             *
;************************************************

	call	Bin2BCD		; Adjusts decimal to BCD
	call	L1homeLCD	; Goto home of line 1
	movf	TempH,W		; Write temperature
	call	putcLCD
	movf	TempL,W
	call	putcLCD
	movlw	'C'		; Also write C for celsius
	call	putcLCD
	goto	Main


;************************************************
;*  Delay20 - delay for exactly 20us including  *
;*  the call and the return instructions.       *
;************************************************
Delay20
;************************************************
;* Place your code here to do the following:    *
;*   - Write code to perform a delay of exactly *
;*     20us including the call and return from  *
;*     this routine.  Use the variable Count to *
;*     help count the number of cycles.         *
;************************************************


;************************************************
;* End of your code                             *
;************************************************
	return

;************************************************
;* Bin2BCD - converts Temp to ASCII             *
;* equivalents.  Stored in TempH:TempL.         *
;************************************************
Bin2BCD
	clrf	TempH		; Initialize variables
	movf	Temp,W
	movwf	TempL
Ttenth				; Subtract 10 from low variable
	movlw	.10		; until result is < 0.  On each
	subwf	TempL,W		; subtraction, increment the
	btfss	STATUS,C	; temperature high byte.
	goto	Over
	movwf	TempL
	incf	TempH,F
	goto	Ttenth
Over
	movlw	0x30		; Make decimal ASCII by adding
	addwf	TempH,F		; '0' to high and low byte
	addwf	TempL,F
	return

	end
